&routine st_order

&rem This routine runs in ARCPLOT/W to calculate stream order of an arc
&rem coverage.
&rem
&rem Assumptions:
&rem 1) the coverage has clean arc and node topology (no overlaps)
&rem 2) every arc's to_node is downstream from its from_node
&rem 3) the only dangling from_nodes are on order 1 streams
&rem 4) the only dangling to_node is on the arc furthest downstream
&rem 5) no braids or splits exist
&rem 6) the arc furthest downstream has its order set to -1
&rem 7) all other arcs have their order set to some other value

&include st_order.inc

&define i -11 &var
&define rec -12 &var

&if &eq "x%-1" "x" &do
   &delim < >
   &type "Usage: &r ST_ORDER [cover] [order_item] {<opt1>/<opt2>}"
   &delim [ ]
   &return
&end
&extract [cover] -1 1
&extract [order_it] -2 1
&if &eq "x%-3" "x" &do
   &sv [option] [opt1]
&else
   &extract [option] -2 1
&end
&value [wksp] WKSP
LINESET COLOR.LIN
CLEARSELECT
RES [cover] ARC [order_it] = -1
SHOW RESELECT [temp] 0
&if &ne [temp] 1 &do
   &type "One arc in stream network (the furthest downstream)"
   &type "must have its order set to -1"
   &return
&end

&rem **** trace stream network
&rem **** and calculate "inverse" order

&type "Tracing stream network..."
LINESYMBOL 5
&sv [i] -1
&while &do
   ARCS [cover]
   CALC %[cover] ARC %[order_it] = %[i]
   &openw [wksp]t$node.lis
   LIST [cover] ARC FNODE_
   &closew
   CLEARSELECT
   &sv [first] .TRUE.
   &open [wksp]t$node.lis error
   &while &do
      &read [temp] [break]
      &extract [node] [temp] 1
      &if &nm [node] &do
         &if &eq [first] .TRUE. &do
            RES [cover] ARC TNODE_ = [node]
            &sv [first] .FALSE.
         &else
            ASEL [cover] ARC TNODE_ = [node]
         &end
      &end
   &end
   &close
   & DEL [wksp]t$node.lis
   SHOW RESELECT [temp] 0
   &if &eq [temp] 0 &do
      &break
   &end
   &dec [i]
&end

&rem **** now drop back down
&rem **** and calculate "true" order

&type "Generating stream order..."
LINESYMBOL 4
&while &ne [i] 0 &do
   CLEARSELECT
   RES [cover] ARC [order_it] = [i]
   &openw [wksp]t$rec.lis
   LIST [cover] ARC $RECNO
   &closew
   &open [wksp]t$rec.lis error
   &while &do
      &read [temp] [break]
      &if &nm [temp] &do
         &cv [rec] [temp]
         CLEARSELECT
         RES [cover] ARC $RECNO IN {[rec]}
         CALC %[cover] ARC [node] = FNODE_
         &r st_calc
         &rv [temp]
         &if &eq [temp] .FALSE. &do
            &return
         &end
         CLEARSELECT
         RES [cover] ARC $RECNO IN {[rec]}
         CALC %[cover] ARC %[order_it] = %[order]
         ARCS [cover]
      &end
   &end
   &close
   & DEL [wksp]t$rec.lis
   &inc [i]
&end
CLEARSELECT
&return

&label error
&type "I/O error"
&close &all
&closew &all
CLEARSELECT
&return

&rem ******************************************************************

&routine st_calc

&include st_order.inc

&define o1 -11 &var
&define o2 -12 &var
&define delta -13 &var
&define sum -14 &var

&value [wksp] WKSP
CLEARSELECT
RES [cover] ARC TNODE_ = [node]
SHOW RESELECT [temp] 0
&if &eq [temp] 0 &do
   &sv [order] 1
   &return .TRUE.
&end
&if &eq [temp] 1 &do
   CALC %[cover] ARC [order] = %[order_it]
   &return .TRUE.
&end
&openw [wksp]t$order.lis
LIST [cover] ARC [order_it]
&closew
&sv [first] .TRUE.
&sv [delta] .FALSE.
&open [wksp]t$order.lis error 2
&while &do
   &read [temp] [break] 2
   &if &nm [temp] &do
      &cv [o2] [temp]
      &if &eq [first] .TRUE. &do
         &value [o1] [o2]
         &value [sum] [o2]
         &sv [first] .FALSE.
         &continue
      &end
      &if &ne [o1] [o2] &do
         &cv [o1] [o1] max [o2]
         &sv [delta] .TRUE.
      &end
      &cv [sum] [sum] + [o2]
   &end
&end
&close 2
& DEL [wksp]t$order.lis
&if &eq [option] [opt1] &do
   &value [order] [sum]
&else
   &if &eq [delta] .TRUE. &do
      &value [order] [o1]
   &else
      &cv [order] [o1] + 1
   &end
&end
&return .TRUE.

&label error
&type "I/O error: st_calc"
&close &all
&closew &all
CLEARSELECT
&return .FALSE.

